*==============================================================================
* Function:			GetDriveSpace
* Purpose:			Returns the total or free space on a drive; workaround for
*						a bug in VFP's DISKSPACE() routine which doesn't handle
*						free space more than 2 GB
* Author:			Doug Hennig, from code written by George Tasker
* Copyright:		(c) 2000 Stonefield Systems Group Inc.
* Last revision:	11/02/2000
* Parameters:		tcDirectory - any directory on the drive to check space for
*						(optional: if it isn't specified, the current drive is
*						used)
*					tlTotal     - .T. to return the total space on the drive or
*						.F. to return the free space
* Returns:			the amount of free or total space, depending on tlTotal, or
*						-1 if an error occurred (such as drive not ready)
* Environment in:	none
* Environment out:	none
*==============================================================================

lparameters tcDirectory, ;
	tlTotal
local lcCaller, ;
	lcTotal, ;
	lcFree, ;
	lcStruct, ;
	lcDir, ;
	lnResult

* Declare a Win32 API function to determine the version.

declare integer GetVersionEx in Win32API ;
	string @lpVersionInformation

* Create the version information structure.

store replicate(chr(0), 8) to lcCaller, lcTotal, lcFree
lcStruct = Decimal2Hex(148) + replicate(chr(0), 144)

* If the path wasn't specified, use the current drive. Otherwise, get the drive
* for the specified path, handling UNC paths specially.

do case
	case vartype(tcDirectory) <> 'C' or empty(tcDirectory)
		lcDir = sys(5)
	case left(tcDirectory, 2) = '\\'
		lcDir = addbs(tcDirectory)
		lcDir = left(lcDir, at('\', lcDir, 4))
	otherwise
		lcDir = addbs(justdrive(tcDirectory))
endcase

* If this is VER_PLATFORM_WIN32_WINDOWS and OSR2 or later, use the
* GetDiskFreeSpaceEx API function to get the drive space.

if GetVersionEx(@lcStruct) <> 0 and ;
	inlist(Hex2Decimal(substr(lcStruct, 17, 4)), 1, 2) and ;
	Hex2Decimal(substr(lcStruct, 13, 4)) > 1000
	declare short GetDiskFreeSpaceEx in Win32API ;
		string @lpDirectoryName, ;
		string @lpFreeBytesAvailableToCaller, ;
		string @lpTotalNumberOfBytes, ;
		string @lpTotalNumberOfFreeBytes
	if GetDiskFreeSpaceEx(@lcDir, @lcCaller, @lcTotal, @lcFree) <> 0
		lnResult = Hex2Decimal(iif(tlTotal, lcTotal, lcFree))
	endif GetDiskFreeSpaceEx(@lcDir, ...

* Since we can't use GetDiskFreeSpaceEx, just use DISKSPACE.

else
	lnResult = diskspace(lcDir)
endif GetVersionEx(@lcStruct) <> 0 ...
return lnResult
